/*
 * LCDDriver.c
 *
 *  Created on: 16.02.2013
 *      Author: kurt
 */

#include "LCDDriverDefinitions.h"
#include "ButtonHandler.h"
#include "LCDDriver.h"
#include "LEDDriver.h"
#include "TouchDriver.h"
#include "avr/io.h"
#include "avr/interrupt.h"
#include "stdio.h"
#include "stdlib.h"
#include "u8g.h"
#include "string.h"

#define MODE_RESETTED 0
#define MODE_PRINTF 1
#define MODE_PRINT_BUTTON 2
#define MODE_LOGO_BUTTON 3

static u8g_t u8g;
static volatile uint8_t LCDmode = MODE_RESETTED;
uint8_t printTarget = 0xff;
int LCD_putchar(char c, FILE *stream);
static FILE LCD_stdout = FDEV_SETUP_STREAM( LCD_putchar, NULL, _FDEV_SETUP_WRITE );

#define LINES 10
#define COLUMNS 40

static uint8_t fontHeight;
static char characterScreen[LINES][COLUMNS + 1];
static uint8_t screenX, screenY;

static int8_t audiogrammData[12];
static int8_t filterData[12];
static uint8_t paintGraphics = 0;

static uint8_t displayBusy = 0;
static uint8_t refreshNeeded = 0;
static uint8_t refreshScheduled = 0;
static uint8_t upperLeft = 0;
static uint8_t lowerRight = 0;

void LCD_init(void) {
	uint8_t scroll;
	stdout = &LCD_stdout;
	screenX = 0;
	screenY = 0;
	for (scroll = 0; scroll < LINES; scroll++) {
		memset(characterScreen[scroll], 0, COLUMNS + 1);
	}
	u8g_InitSPI(&u8g, &u8g_dev_uc1610_dogxl160_2x_bw_hw_spi, PN(1, 0), PN(3, 1), PN(1, 2), PN(1, 1), U8G_PIN_NONE);
	u8g_SetFont(&u8g, u8g_font_6x10);
	fontHeight = u8g_GetFontBBXHeight(&u8g);
	TCCR0A = 0;
	TCCR0B = (1 << CS02) | (1 << CS00);
	TIMSK0 = (1 << TOIE0);
	LCDmode = MODE_RESETTED;
	printTarget = 0;
	refreshNeeded = 1;
	displayBusy = 0;
}

void LCD_setPrintfMode(void) {
	uint8_t scroll;
	screenX = 0;
	screenY = 0;
	for (scroll = 0; scroll < LINES; scroll++) {
		memset(characterScreen[scroll], 0, COLUMNS + 1);
	}
	LCDmode = MODE_PRINTF;
	refreshNeeded = 1;
}

void LCD_setPrintButtonMode(void) {
	Button_init();
	LCDmode = MODE_PRINT_BUTTON;
	refreshNeeded = 1;
}

void LCD_setPrintLogoMode(void) {
	Button_init();
	LCDmode = MODE_LOGO_BUTTON;
	refreshNeeded = 1;
}

void LCD_enableGraphics(uint8_t enable) {
	paintGraphics = enable;
}

void LCD_setGraphicsData(uint8_t where, int8_t audiogramm, int8_t filter) {
	audiogrammData[where] = audiogramm;
	filterData[where] = filter;
	refreshNeeded = 1;
}

void LCD_loop(uint8_t forced) {
	uint8_t objectCount;

	if (!forced && !refreshScheduled) return;
	if (!forced && !refreshNeeded) return;
	displayBusy = 1;
	u8g_FirstPage(&u8g);
	do {
		switch (LCDmode) {
		case MODE_RESETTED:
			u8g_DrawCircle(&u8g, 80, 30, 24, U8G_DRAW_UPPER_LEFT | U8G_DRAW_LOWER_LEFT);
			u8g_DrawCircle(&u8g, 80, 30, 25, U8G_DRAW_UPPER_LEFT | U8G_DRAW_LOWER_LEFT);
			u8g_DrawCircle(&u8g, 80, 60, 24, U8G_DRAW_UPPER_RIGHT | U8G_DRAW_LOWER_RIGHT);
			u8g_DrawCircle(&u8g, 80, 60, 25, U8G_DRAW_UPPER_RIGHT | U8G_DRAW_LOWER_RIGHT);
			u8g_DrawStr(&u8g, 5, 100, "KlarerTon LCD Display 1.0");
			if (upperLeft) {
				u8g_DrawLine(&u8g, 0, 0, 10, 0);
				u8g_DrawLine(&u8g, 0, 0, 0, 10);
			}
			if (lowerRight) {
				u8g_DrawLine(&u8g, 149, 103, 159, 103);
				u8g_DrawLine(&u8g, 159, 103, 159, 93);
			}
			break;
		case MODE_PRINTF:
			for (objectCount = 0; objectCount < LINES; objectCount++) {
				u8g_DrawStr(&u8g, 0, (objectCount + 1) * fontHeight,characterScreen[objectCount]);
			}
			break;
		case MODE_PRINT_BUTTON:
			for (objectCount = 0; objectCount < BUTTON_COUNT; objectCount++) {
				if (buttons[objectCount].buttonID != 0xff) {
					u8g_DrawFrame(&u8g, 40 * (objectCount % 4), objectCount / 4 * 34, 40, 34);
					u8g_DrawStr(&u8g, 40 * (objectCount % 4) + 2, objectCount / 4 * 34 + fontHeight, buttons[objectCount].lines[0]);
					u8g_DrawStr(&u8g, 40 * (objectCount % 4) + 2, objectCount / 4 * 34 + 2 * fontHeight, buttons[objectCount].lines[1]);
					u8g_DrawStr(&u8g, 40 * (objectCount % 4) + 2, objectCount / 4 * 34 + 3 * fontHeight, buttons[objectCount].lines[2]);
				}
			}
			if (paintGraphics > 0) {
				for (objectCount = 0; objectCount < 12; objectCount++) {
					if (objectCount > 0) {
						u8g_DrawLine(&u8g, 10 + 10 * (objectCount - 1), paintGraphics + audiogrammData[objectCount - 1], 10 + 10 * (objectCount), paintGraphics + audiogrammData[objectCount]);
					}
					u8g_DrawLine(&u8g, 10 + 10 * (objectCount), paintGraphics, 10 + 10 * (objectCount), paintGraphics + audiogrammData[objectCount]);
				}
			}
			break;
		case MODE_LOGO_BUTTON:
			u8g_DrawCircle(&u8g, 80, 30, 24, U8G_DRAW_UPPER_LEFT | U8G_DRAW_LOWER_LEFT);
			u8g_DrawCircle(&u8g, 80, 30, 25, U8G_DRAW_UPPER_LEFT | U8G_DRAW_LOWER_LEFT);
			u8g_DrawCircle(&u8g, 80, 60, 24, U8G_DRAW_UPPER_RIGHT | U8G_DRAW_LOWER_RIGHT);
			u8g_DrawCircle(&u8g, 80, 60, 25, U8G_DRAW_UPPER_RIGHT | U8G_DRAW_LOWER_RIGHT);
			for (objectCount = 0; objectCount < BUTTON_COUNT; objectCount++) {
				if (buttons[objectCount].buttonID != 0xff) {
					u8g_DrawFrame(&u8g, 40 * (objectCount % 4), objectCount / 4 * 34, 40, 34);
					u8g_DrawStr(&u8g, 40 * (objectCount % 4) + 2, objectCount / 4 * 34 + fontHeight, buttons[objectCount].lines[0]);
					u8g_DrawStr(&u8g, 40 * (objectCount % 4) + 2, objectCount / 4 * 34 + 2 * fontHeight, buttons[objectCount].lines[1]);
					u8g_DrawStr(&u8g, 40 * (objectCount % 4) + 2, objectCount / 4 * 34 + 3 * fontHeight, buttons[objectCount].lines[2]);
				}
			}
			break;
		}
	} while (u8g_NextPage(&u8g));
	refreshScheduled = 0;
	refreshNeeded = 0;
	displayBusy = 0;
}

uint8_t LCD_isDisplayBusy(void) {
	return displayBusy;
}

void LCD_showUpperLeft(void) {
	upperLeft = 1;
	lowerRight = 0;
	refreshNeeded = 1;
}

void LCD_showLowerRight(void) {
	lowerRight = 1;
	upperLeft = 0;
	refreshNeeded = 1;
}

void LCD_hideCorners(void) {
	lowerRight = 0;
	upperLeft = 0;
	refreshNeeded = 1;
}

void LCD_setCharPos(uint8_t x, uint8_t y) {
	screenX = x;
	screenY = y;
}

void LCD_putBuffer(char c) {
	uint8_t scroll;
	switch (LCDmode) {
	case MODE_RESETTED:
		break;
	case MODE_PRINTF:
		switch(c) {
		case '\r':
			screenX = 0;
			break;
		case '\n':
			screenY++;
			if (screenY == LINES) {
				screenY--;
				for (scroll = 1; scroll < LINES; scroll++) {
					memcpy(characterScreen[scroll - 1], characterScreen[scroll], COLUMNS);
				}
				memset(characterScreen[LINES - 1], ' ', COLUMNS);
			}
			screenX = 0;
			refreshNeeded = 1;
			break;
		case 0x0c:
			screenX = 0;
			screenY = 0;
			for (scroll = 0; scroll < LINES; scroll++) {
				memset(characterScreen[scroll], 0, COLUMNS + 1);
			}
			refreshNeeded = 1;
			break;
		default:
			if (screenX < COLUMNS) {
				characterScreen[screenY][screenX++] = c;
				refreshNeeded = 1;
			}
			break;
		}
		break;
		case MODE_PRINT_BUTTON:
		case MODE_LOGO_BUTTON:
			if (buttons[printTarget].line < BUTTON_COUNT && buttons[printTarget].column < BUTTON_TEXTSIZE) {
				buttons[printTarget].lines[buttons[printTarget].line][buttons[printTarget].column++] = c;
				refreshNeeded = 1;
			}
			break;
	}
}

int LCD_putchar(char c, FILE* stream) {
	LCD_putBuffer(c);
	return 0;
}

ISR(TIMER0_OVF_vect) {
	refreshScheduled = 1;
}

